import promiseRetry from '../retry';

/**
 * 装饰器函数：给异步函数增加重试
 * @param {Object} settings
 * @param {number} [settings.retries = 5]
 * @param {number} [settings.timeout = 2000]
 * @param {Function} [settings.onError]
 * @param {Function} [settings.onRetrying]
 * @param {Function} [settings.onRetryFailed]
 * @example
 * class LocalStream {
 *   @promiseRetryDecorator({
 *     retries: 10,
 *     timeout: 3000,
 *     onRetryFailed: function(error) {
 *     }
 *   })
 *   async recoverCapture(options) {}
 * }
 */
export function promiseRetryDecorator(settings = { retries: 5, timeout: 2000 }) {
  return function(target, name, descriptor) {
    const oldFn = promiseRetry({
      retryFunction: descriptor.value,
      settings,
      onError: settings.onError,
      onRetrying: settings.onRetrying,
      onRetryFailed: settings.onRetryFailed
    });

    descriptor.value = function(...args) {
      return oldFn.apply(this, args);
    };
    return descriptor;
  };
}
